@@ -3,8 +3,8 @@ import fnmatch
 import logging
 import pathlib
 import re
+from collections import OrderedDict
 from collections.abc import Mapping
-from collections.abc import MutableMapping
 from collections.abc import Sequence
 from typing import Any
 from typing import Dict
@@ -15,7 +15,7 @@ from typing import Set
 
 import aiofiles
 import msgpack
-import yaml
+import yaml.representer
 
 from . import args
 from . import DEFAULT_TARGET_DELIM
@@ -23,19 +23,38 @@ from . import DEFAULT_TARGET_DELIM
 log = logging.getLogger(__name__)
 
 
-class CaseInsensitiveDict(MutableMapping):
+class DictMeta(type):
+    """
+    Metaclass for Dict types so that they pass isinstance checks
+    """
+
+    def __instancecheck__(self, other) -> bool:
+        if not hasattr(other, "__iter__"):
+            return False
+        if not hasattr(other, "__getitem__"):
+            return False
+        if not hasattr(other, "__setitem__"):
+            return False
+        if not hasattr(other, "__len__"):
+            return False
+        return True
+
+
+class CaseInsensitiveDict(metaclass=DictMeta):
     """
     Inspired by requests' case-insensitive dict implementation, but works with
     non-string keys as well.
     """
 
+    __metaclass__ = DictMeta
+
     def __init__(self, init=None, **kwargs):
         """
         Force internal dict to be ordered to ensure a consistent iteration
         order, irrespective of case.
         """
         self._data = {}
-        self.update(init or {}, **kwargs)
+        self._data.update(init or {}, **kwargs)
 
     def __len__(self):
         return len(self._data)
@@ -135,9 +154,9 @@ class ImmutableDict(Mapping):
         return repr(copy.copy(self))
 
 
-class NamespaceDict(dict):
+class NamespaceDict(OrderedDict):
     """
-    A dictionary that can access it's string keys through the namespace
+    A dictionary that can access its string keys through the namespace
     """
 
     def __init__(self, seq: Mapping = None, **kwargs):
@@ -153,45 +172,26 @@ class NamespaceDict(dict):
             in the keyword argument list.  For example:  NamespaceDict(one=1, two=2)
         """
         if seq:
-            if isinstance(seq, Mapping):
-                super().__init__()
-                for k, v in seq.items():
-                    self[k] = v
-            else:
-                super().__init__(seq)
+            super().__init__(seq)
         else:
             super().__init__()
 
-        for k, v in kwargs.items():
-            self[k] = v
-
-    def __setitem__(self, k, v):
-        if isinstance(v, Mapping) and not isinstance(v, NamespaceDict):
-            v = NamespaceDict(v)
-            self[k] = v
-        else:
-            super().__setitem__(k, v)
+        self.update(**kwargs)
 
     def __setattr__(self, k: str, v: Any):
-        if isinstance(v, Mapping) and not isinstance(v, NamespaceDict):
-            v = NamespaceDict(v)
         self[k] = v
 
     def __getattr__(self, k: str):
         if k.startswith("_"):
             return super().__getattribute__(k)
-        return self[k]
-
-    def __copy__(self):
-        return {k: copy.copy(v) for k, v in self.items()}
-
-    def __deepcopy__(self, memodict=None):
-        if memodict is None:
-            memodict = {}
-        return NamespaceDict(copy.deepcopy(self.copy(), memodict))
+        value = self[k]
+        # Allow chain-getting namespaced values
+        if isinstance(value, Mapping):
+            return NamespaceDict(value)
+        return value
 
 
-class SafeNamespaceDict(NamespaceDict):
+class SafeNamespaceDict(NamespaceDict, metaclass=DictMeta):
     """
     A Dict that does not expose values when represented as a string
     """
@@ -203,12 +203,12 @@ class SafeNamespaceDict(NamespaceDict):
         return repr(self)
 
 
-class FileDict(Mapping):
+class FileDict(metaclass=DictMeta):
     """
     A dictionary type that stores it's information in a file
     """
 
-    def __init__(self, cache: str, seq: Iterable = None, **kwargs):
+    def __init__(self, cache: str, seq: Mapping = None, **kwargs):
         self.cache = pathlib.Path(cache)
         if seq:
             for k in seq:
